==> pam_usb-0.3.1.tar.gz: pam_usb-0.3.1/src/dsa.c <==
/*
**  Copyright (c) 2003,2004 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "pam.h"
#include "conf.h"
#include "dsa.h"

int prompt_for_password (char *passphrase, int size, int rwflag, void *u)
{
	pam_handle_t *pamh = (pam_handle_t *)u;
	char *buf;
	int len;

	converse (pamh, PAM_TEXT_INFO,
		  "Your private key is locked using symmetric encryption");

	buf = converse (pamh, PAM_PROMPT_ECHO_OFF,
			"Passphrase to unlock the key: ");

	len = strlen (buf);

	if (len <= 0)
		return 0;
	if (len > size)
		len = size;

	memcpy (passphrase, buf, len);
	return len;
}

DSA *import_public_key (FILE *f)
{
	DSA *key;

	if (!(key = PEM_read_DSA_PUBKEY (f, NULL, NULL, NULL)))
		return NULL;

	return key;
}

DSA *import_private_key (FILE *f, pam_handle_t *pamh)
{
	DSA *key;

	OpenSSL_add_all_algorithms();

	if (!(key = PEM_read_DSAPrivateKey (f, NULL, prompt_for_password,
					    (void*)pamh)))
		return NULL;

	return key;
}


int valid_dsa_keys (DSA *private, DSA *public)
{
	unsigned char rdata[20];
	unsigned char sig[256];
	unsigned int siglen;
	int i;

	DEBUGP ("Checking DSA key pair...\n");

	for (i=0; i < usb_conf.sign_times; ++i) {

		memset (rdata, 0, sizeof rdata);
		if (RAND_pseudo_bytes (rdata, sizeof rdata) < 0) {
			DEBUGP ("cannot generate pseudo random data\n");
			return 0;
		}

		DEBUGP ("Signing pseudo random data [%d time(s)]...\n", (i+1));

		DSA_sign (0, rdata, sizeof rdata, sig, &siglen, private);
		if (DSA_verify (0, rdata, sizeof rdata, sig, siglen, public) != 1) {
			DEBUGP ("Invalid signature\n");
			return 0;
		}
		DEBUGP ("Valid signature\n");
	}

	if (BN_cmp (private->pub_key, public->pub_key) != 0) {
		DEBUGP ("BNs mismatched\n");
		return 0;
	}

	return 1;
}

==> pam_usb-0.3.2.tar.gz: pam_usb-0.3.2/src/dsa.c <==
/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "pam.h"
#include "conf.h"
#include "dsa.h"
#include "auth.h"

int prompt_for_password (char *passphrase, int size, int rwflag, void *u)
{
	pam_handle_t *pamh = (pam_handle_t *)u;
	const char *pass;
	int len;

	DEBUGP ("Private key is encrypted\n");

	if (!pam_get_pass (pamh, &pass, "Passphrase to unlock the key: "))
		return 0;

	len = strlen (pass);

	if (len <= 0)
		return 0;
	if (len > size)
		len = size;

	memcpy (passphrase, pass, len);
	return len;
}

DSA *import_public_key (FILE *f)
{
	DSA *key;

	if (!(key = PEM_read_DSA_PUBKEY (f, NULL, NULL, NULL)))
		return NULL;

	return key;
}

DSA *import_private_key (FILE *f, pam_handle_t *pamh)
{
	DSA *key;

	OpenSSL_add_all_algorithms();

	if (!(key = PEM_read_DSAPrivateKey (f, NULL, prompt_for_password,
					    (void*)pamh)))
		return NULL;

	return key;
}


int valid_dsa_keys (DSA *private, DSA *public)
{
	unsigned char rdata[20];
	unsigned char sig[256];
	unsigned int siglen;
	int i;

	DEBUGP ("Checking DSA key pair...\n");

	for (i=0; i < usb_conf.sign_times; ++i) {

		memset (rdata, 0, sizeof rdata);
		if (RAND_pseudo_bytes (rdata, sizeof rdata) < 0) {
			DEBUGP ("cannot generate pseudo random data\n");
			return 0;
		}

		DEBUGP ("Signing pseudo random data [%d time(s)]...\n", (i+1));

		DSA_sign (0, rdata, sizeof rdata, sig, &siglen, private);
		if (DSA_verify (0, rdata, sizeof rdata, sig, siglen, public) != 1) {
			DEBUGP ("Invalid signature\n");
			return 0;
		}
		DEBUGP ("Valid signature\n");
	}

	if (BN_cmp (private->pub_key, public->pub_key) != 0) {
		DEBUGP ("BNs mismatched\n");
		return 0;
	}

	return 1;
}
